// Arduino stepper motor control code
// version 02 for nautilus clock with Teensy processor
//    adds RTC, hand position sensor, button to advance hand (set clock),
//    and logic to sync hand with real time

#include <Stepper.h> // Include the header file
#include <TimeLib.h>
#include <DS1307RTC.h>  // a basic DS1307 library that returns time as a time_t

// change this to the number of steps on your motor
#define STEPS 32

// create an instance of the stepper class using the steps and pins
// on driver board IN  1   2  3   4
Stepper stepper(STEPS, 8, 10, 9, 11);
//Stepper stepper(STEPS, 2, 4, 3, 5); // nano
//Stepper stepper(STEPS, 10, 12, 11, 13); // 

volatile boolean syncPosition = false;  

#define rev 2048  //steps for full rev
#define minutSteps 34  //# steps (s/b 34.1333)
#define handSync 36
// position of minute hand at sync interrupt

#define handPosPin A0
#define minSetPin  A9 
#define hourSetPin A8

int handMin = 0;  //keep track of where is minute hand
int newMin = 0;
int nowHour = 0;  // starts at 0, incremented when min > 59
int nowMin = 0;
unsigned long lastMin = 0;  //minute from RTC
unsigned long lastSec = 0;  //ms timestamp
tmElements_t tm;

    // pins for hour LEDs
// hour LED pins 12,01,02,03,04,05,06,07,08,09,10,11    
int hourPin[] = {A2, 4, 6, 8, 3, 5, 2, 7,A1,A3,A4,A5};  //not sure why shift needed

void setup() {
  Serial.begin(9600);
  stepper.setSpeed(600);

  
  pinMode (handPosPin, INPUT_PULLUP);  // put +v on pin to prevent float voltage
  pinMode(hourSetPin,INPUT_PULLUP);  
  pinMode(minSetPin,INPUT_PULLUP);  

  for (int x=0;x<12;x++) {
    pinMode (hourPin[x], OUTPUT);
    digitalWrite(hourPin[x], LOW);   //turned off
  }
  
//  adjustTime(60*60*23); // set it back one hr to 12
  setHourLED();     //set the correct hour LED
  nowHour = hour();    //starts at hour 0

}

#define hallThresh 900  // level abouve which indicates magnet is seen
void loop() {

  // allow button to advance minute hand and counter
  if (minButton() == true) {  // if button pushed
    adjustTime(60); // seconds adj in Arduino clock
    lastMin = minute();  //reset to elapsed doesn't trigger
    stepper.step(minutSteps);  // move the hand
    handMin++; if (handMin>50) handMin = 0; //update hand minute
    for (int x=0;x<12;x++) {  //
      digitalWrite(hourPin[x], HIGH);   //flash all leds to show button pushed
    }
    Serial.print("min adj to ");Serial.println(handMin);
    delay(50);
    setHourLED();  //to turn off most leds
    return;
  }

  // allow button to advance hour counter
  if (hourButton() == true) {  // if button pushed
    nowHour++;          //update hour number
    if (nowHour > 11) nowHour = 0; 
    setHourLED();     //set the correct hour LED
    Serial.print("hour adj to ");Serial.println(nowHour);
    delay(50);
    return;
  }
        
        // see if we're at the sync point
  int testVal = analogRead(handPosPin);  
  if (testVal > hallThresh) {  //it sees magnet (every hour)
    if (syncPosition == false)  {  // and it's the first time
      syncPosition = true;
      int minuteDiff = handMin - handSync; //compare to set point
       // see if hands need adjusting
      if (minuteDiff > 1) {         //minute hand too far ahead
        Serial.print("hand position adj by=");Serial.println(minuteDiff);
        stepper.step(minutSteps * minuteDiff);  // move it up 
      } else if (minuteDiff < -1) { //minute hand too far behind 
        Serial.print("hand position adj min by=");Serial.println(minuteDiff);
        stepper.step(minutSteps * minuteDiff);   // move it back 
      }
      handMin = handSync;  //keep track of where hand is
    }  
  } else {
    syncPosition = false;  // no more magnet, turn off sw
  }

  
  //  check to see if a minute has elapsed
  newMin = minute();  //
  if (newMin != lastMin) {  //use arduino minute only to see if elapsed
  // if so, move one rotation plus 34 steps
    stepper.step(rev + minutSteps);
    lastMin = newMin;  //use the minute from arduino clock
    handMin++;          // 
    if (handMin > 59) {   //see if hour need update
      handMin = 0;        //keep track of hand minute
      nowHour++;          //update hour number
      if (nowHour > 11) nowHour = 0; 
      setHourLED();       //update the hour LED
    }
    Serial.print("minute=");Serial.println(newMin);
  } else if (millis() - lastSec >=1000) {  //keep jiggling to show alive
    lastSec = millis();
    stepper.step(minutSteps);
    delay(400);
    stepper.step(-minutSteps);
  }

/*  if (Serial.available()>0)  //allow manual adjustment via console
  {
    int val = Serial.parseInt();
    Serial.println(val); //for debugging
    int stps = Serial.parseInt();

    adjustTime(60*60); // seconds adj in Arduino clock
    setHourLED();     //set the correct hour LED
    Serial.print("hour is ");Serial.println(hour());
    
//    Serial.println(stps); //for debugging
//    stepper.step(val);
  } */
//    int wkpin = digitalRead(hourSetPin);
//    Serial.print("hourButton() is ");Serial.print(hourButton());
//    wkpin = digitalRead(minSetPin);
//    Serial.print("minButton() is ");Serial.println(minButton());

  
  
}

void printTime() {
  // digital clock display of the time
  Serial.print(hour());
  printDigits(minute());  //with colons etc
  printDigits(second());
  Serial.println();
}

void printDigits(int digits){
  // utility function for digital clock display: prints preceding colon and leading 0
  Serial.print(":");
  if(digits < 10)
    Serial.print('0');
  Serial.print(digits);
}


void setHourLED() {
  for (int x=0;x<12;x++) {  //turn em all off
    digitalWrite(hourPin[x], LOW);   //turned off
  }
  digitalWrite(hourPin[nowHour], HIGH);   //
}

bool hourButton() {   // see if switch is closed
      // this is now the only place in code touchPin is checked
  if (digitalRead(hourSetPin) == LOW) {  //sw closed
    delay(75);  //wait a tick  and check it again
    if (digitalRead(hourSetPin) == LOW) return true;  // it's really closed
  }
  return false;  // this ignores spurious closes
}

bool minButton() {   // see if switch is closed
      // this is now the only place in code touchPin is checked
  if (digitalRead(minSetPin) == LOW) {  //sw closed
    delay(75);  //wait a tick  and check it again
    if (digitalRead(minSetPin) == LOW) return true;  // it's really closed
  }
  return false;  // this ignores spurious closes
}
